#!/usr/bin/env python

import sys, os, os.path, shutil, subprocess, glob, re
from nacoma.hooks import Change

reportable_types = ['host', 'service']

def find_pnp_dir():
    return '/opt/monitor/op5/pnp/perfdata'

def clean_name(name):
    '''this corresponds to the cleanup sub in process_perfdata.pl'''
    return re.sub(r'[ :\/]', r'_', name)

def cleanup(name):
    name = clean_name(name)
    dir = find_pnp_dir()
    try:
        os.mkdir(os.path.join(dir, '.trash'))
    except OSError:
        # already exists
        pass
    try:
        if ';' in name:
            hostname, servicename = name.split(';')
            try:
                os.mkdir(os.path.join(dir, '.trash', hostname), 0775)
            except OSError:
                pass
            try:
                os.remove(os.path.join(dir, '.trash', hostname, servicename+'.xml'))
                for rrd in glob.glob(os.path.join(dir, '.trash', hostname, servicename+'*.rrd')):
                    os.remove(rrd)
            except OSError:
                pass
            shutil.move(os.path.join(dir, hostname, servicename+'.xml'), os.path.join(dir, '.trash', hostname, servicename+'.xml'))
            for rrd in glob.glob(os.path.join(dir, hostname, servicename+'*.rrd')):
                shutil.move(rrd, os.path.join(dir, '.trash', hostname, os.path.basename(rrd)))
        else:
            try:
                shutil.rmtree(os.path.join(dir, '.trash', name))
            except OSError:
                pass
            shutil.move(os.path.join(dir, name), os.path.join(dir, '.trash', name))
    except IOError, e:
        if e.errno != 2: # 2 => 'no such file or directory'
            print "IO error when doing cleanup"
            print e
        return

def move_host(fromname, toname):
    dir = find_pnp_dir()
    frompath = os.path.join(dir, clean_name(fromname))
    topath = os.path.join(dir, clean_name(toname))
    try:
        shutil.move(frompath, topath)
    except IOError, e:
        if e.errno != 2: # 2 => 'no such file or directory'
            print "IO error when moving host files"
            print e
        return
    for filepath in glob.glob(topath + '/*.xml'):
        output = subprocess.Popen(['/bin/sed',
            filepath,
            '-i',
            '-e', r's|%s|%s|' % (frompath, topath)], # important to use correct RRD database
            stdout=subprocess.PIPE).communicate()[0]

def move_service(fromname, toname):
    # this assumes that a service cannot change both host and name at once.
    # currently true, at least...
    hostname, fromname = fromname.split(';')
    toname = toname.split(';')[1]
    dir = find_pnp_dir()
    frompath = os.path.join(dir, clean_name(hostname), clean_name(fromname))
    topath = os.path.join(dir, clean_name(hostname), clean_name(toname))
    try:
        shutil.move(frompath + '.rrd', topath + '.rrd')
        shutil.move(frompath + '.xml', topath + '.xml')
    except IOError, e:
        if e.errno != 2: # 2 => 'no such file or directory'
            print "IO error when moving host files"
            print e
        return
    output = subprocess.Popen(['/bin/sed',
        topath + '.xml',
        '-i',
        '-e', r's|%s|%s|' % (frompath, topath)], # important to use correct RRD database
        stdout=subprocess.PIPE).communicate()[0]


for line in sys.stdin:
    change = Change(line)
    if change.type not in reportable_types:
        continue

    if change.is_renamed():
        cleanup(change.newname)
        if change.type == 'host':
            move_host(change.oldname, change.newname)
        else:
            move_service(change.oldname, change.newname)
    elif change.is_deleted():
        cleanup(change.oldname)
        pass
