<?php
/**
 * Test static methods in datamodel_Core helper class.
 */
class datamodel_helper_Test extends PHPUnit_Framework_TestCase {

	const XML_TEST_FILENAME = '/tmp/datamodeltest.xml';

	public function setUp() {
		require_once(__DIR__  . '/../application/helpers/datamodel.php');
	}

	public function test_filter_host_by_page() {
		// Page graph test data
		$page_graph = array(
			0 => array(
				"host_name"    => "En host",
				"service_desc" => "min nya service"
			),
			1 => array(
				"host_name"    => "8436test1",
				"service_desc" => "_HOST_"
			),
		);

		// Assert filter_host_by_page works with a "normal" host name.
		$hostname = datamodel_Core::filter_host_by_page(
			$page_graph,
			'8436test1',
			array('datamodel_Core', 'match_string')
		);
		$this->assertEquals('8436test1', $hostname);

		// Assert filter_host_by_page works with a host name that contains spaces.
		$hostname = datamodel_Core::filter_host_by_page(
			$page_graph,
			'En_host',
			array('datamodel_Core', 'match_string')
		);

		$this->assertEquals('En_host', $hostname);
	}

	public function test_filter_service_by_graph() {

		$graph_data = array(
			"host_name"    => "En host",
			"service_desc" => "min nya service"
		);

		$matching_data = datamodel_Core::filter_service_by_graph(
			$graph_data,
			'En_host',
			'min_nya_service',
			array('datamodel_Core', 'match_string')
		);

		$expected_data = array(
			'service_desc' => 'min_nya_service',
			'host_name'    => 'En_host',
			'source'       => null
		);
		$this->assertSame($expected_data, $matching_data);
	}

	public function test_read_xml() {

		$xml_file_content =
			'<?xml version="1.0" encoding="UTF-8" standalone="yes"?>' . "\n" .
			'<NAGIOS>' .
			'  <DATASOURCE>'	.
			'    <TEMPLATE>check-host-alive</TEMPLATE>' .
			'    <RRDFILE>Du__och__jag_pkt.rrd</RRDFILE>' .
			'    <RRD_STORAGE_TYPE>MULTIPLE</RRD_STORAGE_TYPE>' .
			'    <RRD_HEARTBEAT>8460</RRD_HEARTBEAT>' .
			'    <IS_MULTI>0</IS_MULTI>' .
			'    <DS>1</DS>' .
			'    <NAME>pkt</NAME>' .
			'    <LABEL>pkt</LABEL>' .
			'    <UNIT/>' .
			'    <ACT>1</ACT>' .
			'    <WARN>0</WARN>' .
			'    <WARN_MIN/>' .
			'    <WARN_MAX/>' .
			'    <WARN_RANGE_TYPE/>' .
			'    <CRIT>0</CRIT>' .
			'    <CRIT_MIN/>' .
			'    <CRIT_MAX/>' .
			'    <CRIT_RANGE_TYPE/>' .
			'    <MIN>0</MIN>' .
			'    <MAX>5</MAX>' .
			'  </DATASOURCE>' .
			'  <RRD>' .
			'    <RC>0</RC>' .
			'    <TXT>successful updated</TXT>' .
			'  </RRD>' .
			'  <NAGIOS_AUTH_HOSTNAME>127.0.0.1</NAGIOS_AUTH_HOSTNAME>' .
			'  <NAGIOS_AUTH_SERVICEDESC>Du  och  jag</NAGIOS_AUTH_SERVICEDESC>' .
			'  <NAGIOS_CHECK_COMMAND>check-host-alive</NAGIOS_CHECK_COMMAND>' .
			'  <NAGIOS_DATATYPE>SERVICEPERFDATA</NAGIOS_DATATYPE>' .
			'  <NAGIOS_DISP_HOSTNAME>127.0.0.1</NAGIOS_DISP_HOSTNAME>' .
			'  <NAGIOS_DISP_SERVICEDESC>Du  och  jag</NAGIOS_DISP_SERVICEDESC>' .
			'  <NAGIOS_HOSTNAME>127.0.0.1</NAGIOS_HOSTNAME>' .
			'  <NAGIOS_HOSTSTATE>UP</NAGIOS_HOSTSTATE>' .
			'  <NAGIOS_HOSTSTATETYPE>HARD</NAGIOS_HOSTSTATETYPE>' .
			'  <NAGIOS_MULTI_PARENT/>' .
			'  <NAGIOS_PERFDATA>pkt=1;0;0;0;5 rta=0.016;2000.000;2000.000;;' .
				' pl=0%;95;100;; </NAGIOS_PERFDATA>' .
			'  <NAGIOS_RRDFILE/>' .
			'  <NAGIOS_SERVICECHECKCOMMAND>check-host-alive' .
				'</NAGIOS_SERVICECHECKCOMMAND>' .
			'  <NAGIOS_SERVICEDESC>Du__och__jag</NAGIOS_SERVICEDESC>' .
			'  <NAGIOS_SERVICEPERFDATA>pkt=1;0;0;0;5 rta=0.016;2000.000;' .
				'2000.000;; pl=0%;95;100;;</NAGIOS_SERVICEPERFDATA>' .
			'  <NAGIOS_SERVICESTATE>OK</NAGIOS_SERVICESTATE>' .
			'  <NAGIOS_SERVICESTATETYPE>HARD</NAGIOS_SERVICESTATETYPE>' .
			'  <NAGIOS_TIMET>1451921274</NAGIOS_TIMET>' .
			'  <NAGIOS_XMLFILE>Du__och__jag.xml</NAGIOS_XMLFILE>' .
			'  <XML>' .
			'    <VERSION>4</VERSION>' .
			'  </XML>' .
			"</NAGIOS>\n";

		$expected_ds = array(array(
			'TEMPLATE' =>         'check-host-alive',
			'RRDFILE' =>          'Du__och__jag_pkt.rrd',
			'RRD_STORAGE_TYPE' => 'MULTIPLE',
			'RRD_HEARTBEAT' =>    '8460',
			'IS_MULTI' =>         '0',
			'DS' =>               '1',
			'NAME' =>             'pkt',
			'LABEL' =>            'pkt',
			'UNIT' =>             '',
			'ACT' =>              '1',
			'WARN' =>             '0',
			'WARN_MIN' =>         '',
			'WARN_MAX' =>         '',
			'WARN_RANGE_TYPE' =>  '',
			'CRIT' =>             '0',
			'CRIT_MIN' =>         '',
			'CRIT_MAX' =>         '',
			'CRIT_RANGE_TYPE' =>  '',
			'MIN' =>              '0',
			'MAX' =>              '5'
		));

		$expected_macro = array(
			'AUTH_SERVICEDESC' =>    'Du  och  jag',
			'AUTH_HOSTNAME' =>       '127.0.0.1',
			'CHECK_COMMAND' =>       'check-host-alive',
			'DATATYPE' =>            'SERVICEPERFDATA',
			'DISP_HOSTNAME' =>       '127.0.0.1',
			'DISP_SERVICEDESC' =>    'Du  och  jag',
			'HOSTNAME' =>            '127.0.0.1',
			'HOSTSTATE' =>           'UP',
			'HOSTSTATETYPE' =>       'HARD',
			'MULTI_PARENT' =>        '',
			'PERFDATA' =>
				'pkt=1;0;0;0;5 rta=0.016;2000.000;2000.000;; pl=0%;95;100;; ',
			'RRDFILE' =>             '',
			'SERVICECHECKCOMMAND' => 'check-host-alive',
			'SERVICEDESC' =>         'Du__och__jag',
			'SERVICEPERFDATA' =>
				'pkt=1;0;0;0;5 rta=0.016;2000.000;2000.000;; pl=0%;95;100;;',
			'SERVICESTATE' =>        'OK',
			'SERVICESTATETYPE' =>    'HARD',
			'TIMET' =>               '1451921274',
			'XMLFILE' =>             'Du__och__jag.xml'
		);

		file_put_contents(self::XML_TEST_FILENAME, $xml_file_content);

		$ds = array();
		$macro = array();
		$xml_data = datamodel_Core::read_xml(self::XML_TEST_FILENAME, $ds, $macro);

		$this->assertSame($expected_ds, $ds);
		$this->assertSame($expected_macro, $macro);
		$this->assertSame($xml_file_content, $xml_data->asXML());
	}

	public function test_match_regex() {
		$match_res
			= datamodel_Core::match_regex('jagjagheter123', '^jagjag.*123$');
		$this->assertTrue($match_res);

		$match_res
			= datamodel_Core::match_regex('nujagjagheter123', '^jagjag.*123$');
		$this->assertFalse($match_res);
	}

	public function test_match_string() {
		$match_res = datamodel_Core::match_string(
			'var en gång', 'det, var en gång, en liten gumma'
		);
		$this->assertTrue($match_res);

		$match_res = datamodel_Core::match_string(
			'detta finns inte i texten', 'det var en gång en liten gumma'
		);
		$this->assertFalse($match_res);
	}
}
